//
//  AppDelegate.swift
//  DisplayingGlances
//
//  Created by Wei-Meng Lee on 29/4/15.
//  Copyright (c) 2015 Wei-Meng Lee. All rights reserved.
//

import UIKit

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?
    
    //---convert from NSDate format to String---
    func currentDateToString() -> String {
        var formatter: NSDateFormatter = NSDateFormatter()
        formatter.dateFormat = "yyyy-MM-dd HH:mm:ss zzz"
        return formatter.stringFromDate(NSDate())
    }
    
    //---extract the required data from the JSON string---
    func parseJSONData(data: NSData) {
        var error: NSError?
        
        var parsedJSONData = NSJSONSerialization.JSONObjectWithData(
            data, options: NSJSONReadingOptions.allZeros, error: &error) as!
            [String:AnyObject]
        
        var query = parsedJSONData["query"] as! [String:AnyObject]
        
        if let results = query["results"] as? [String:AnyObject] {
            if let quotes = results["quote"] as? [[String:AnyObject]] {
                var defaults = NSUserDefaults(suiteName: "group.learningwatchkit.displayingglances.app")
                for stock in quotes {
                    //---stock symbol---
                    var symbol = stock["symbol"] as! String
                    
                    //---save the asking price and date it was fetched as a dictionary---
                    var value = ["ask": stock["Ask"] as! String, "date": currentDateToString()]
                    
                    //---save the stock details into the dictionary---
                    defaults?.setObject(value, forKey: symbol)
                }
                defaults?.synchronize()
            }
        }
    }
    
    //---performing a background fetch---
    func application(application: UIApplication, performFetchWithCompletionHandler completionHandler: (UIBackgroundFetchResult) -> Void) {
        var urlString = "http://query.yahooapis.com/v1/public/yql?q=select%20*%20from%20yahoo.finance.quotes%20where%20symbol%20in%20(%22AAPL%22%2C%22MSFT%22)%0A%09%09&env=http%3A%2F%2Fdatatables.org%2Falltables.env&format=json"
        
        var session = NSURLSession.sharedSession()
        session.dataTaskWithURL(NSURL(string:urlString)!,
            completionHandler: {
                (data, response, error) -> Void in
                var httpResp = response as! NSHTTPURLResponse
                if error == nil && httpResp.statusCode == 200 {
                    //---parse the JSON result---
                    self.parseJSONData(data)
                    completionHandler(UIBackgroundFetchResult.NewData)
                } else {
                    completionHandler(UIBackgroundFetchResult.Failed)
                }
        }).resume()
    }
    
    //---temporary fix for Xcode 6's bug in simulating a background fetch---
    func temporaryFixForBackgroundFetchOnSimulator() {
        var defaults = NSUserDefaults(suiteName:
            "group.learningwatchkit.displayingglances.app")
        defaults?.setObject(["ask":"131.89", "date":currentDateToString()], forKey: "AAPL")
        defaults?.synchronize()
    }
    
    func application(application: UIApplication, didFinishLaunchingWithOptions launchOptions: [NSObject: AnyObject]?) -> Bool {
        // Override point for customization after application launch.
        UIApplication.sharedApplication().setMinimumBackgroundFetchInterval(
            UIApplicationBackgroundFetchIntervalMinimum)
        
        //========
        temporaryFixForBackgroundFetchOnSimulator()
        //========
        
        return true
    }

    func applicationWillResignActive(application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(application: UIApplication) {
        // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }


}

